@@ -10,7 +10,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from openstack.compute import compute_service
 from openstack import resource
 from openstack import utils
 
@@ -20,11 +19,9 @@ class Service(resource.Resource):
     resources_key = 'services'
     base_path = '/os-services'
 
-    service = compute_service.ComputeService()
-
     # capabilities
     allow_list = True
-    allow_update = True
+    allow_commit = True
 
     # Properties
     #: Status of service
