@@ -13,15 +13,16 @@
 # under the License.
 
 import copy
-import math
 import warnings
 
-from keystoneauth1 import adapter
+from keystoneauth1 import discover
 import keystoneauth1.exceptions.catalog
 from keystoneauth1 import session as ks_session
+import os_service_types
 import requestsexceptions
 from six.moves import urllib
 
+from openstack import _adapter
 from openstack import version as openstack_version
 from openstack import _log
 from openstack.config import _util
@@ -37,6 +38,18 @@ def _make_key(key, service_type):
         return "_".join([service_type, key])
 
 
+def _get_implied_microversion(version):
+    if not version:
+        return
+    if '.' in version:
+        # Some services historically had a .0 in their normal api version.
+        # Neutron springs to mind with version "2.0". If a user has "2.0"
+        # set in a variable or config file just because history, we don't
+        # need to send any microversion headers.
+        if version.split('.')[1] != "0":
+            return version
+
+
 def from_session(session, name=None, region_name=None,
                  force_ipv4=False,
                  app_name=None, app_version=None, **kwargs):
@@ -104,6 +117,8 @@ class CloudRegion(object):
         self._cache_arguments = cache_arguments
         self._password_callback = password_callback
 
+        self._service_type_manager = os_service_types.ServiceTypes()
+
     def __getattr__(self, key):
         """Return arbitrary attributes."""
 
@@ -159,21 +174,26 @@ class CloudRegion(object):
 
     def get_requests_verify_args(self):
         """Return the verify and cert values for the requests library."""
-        if self.config['verify'] and self.config['cacert']:
-            verify = self.config['cacert']
+        insecure = self.config.get('insecure', False)
+        verify = self.config.get('verify', True)
+        cacert = self.config.get('cacert')
+        # Insecure is the most aggressive setting, so it wins
+        if insecure:
+            verify = False
+        if verify and cacert:
+            verify = cacert
         else:
-            verify = self.config['verify']
-            if self.config['cacert']:
+            if cacert:
                 warnings.warn(
                     "You are specifying a cacert for the cloud {full_name}"
                     " but also to ignore the host verification. The host SSL"
                     " cert will not be verified.".format(
                         full_name=self.full_name))
 
-        cert = self.config.get('cert', None)
+        cert = self.config.get('cert')
         if cert:
-            if self.config['key']:
-                cert = (cert, self.config['key'])
+            if self.config.get('key'):
+                cert = (cert, self.config.get('key'))
         return (verify, cert)
 
     def get_services(self):
@@ -189,46 +209,113 @@ class CloudRegion(object):
     def get_auth_args(self):
         return self.config.get('auth', {})
 
+    def _get_config(
+            self, key, service_type,
+            default=None,
+            fallback_to_unprefixed=False,
+            converter=None):
+        '''Get a config value for a service_type.
+
+        Finds the config value for a key, looking first for it prefixed by
+        the given service_type, then by any known aliases of that service_type.
+        Finally, if fallback_to_unprefixed is True, a value will be looked
+        for without a prefix to support the config values where a global
+        default makes sense.
+
+        For instance, ``_get_config('example', 'block-storage', True)`` would
+        first look for ``block_storage_example``, then ``volumev3_example``,
+        ``volumev2_example`` and ``volume_example``. If no value was found, it
+        would look for ``example``.
+
+        If none of that works, it returns the value in ``default``.
+        '''
+        if service_type is None:
+            return self.config.get(key)
+
+        for st in self._service_type_manager.get_all_types(service_type):
+            value = self.config.get(_make_key(key, st))
+            if value is not None:
+                break
+        else:
+            if fallback_to_unprefixed:
+                value = self.config.get(key)
+
+        if value is None:
+            return default
+        else:
+            if converter is not None:
+                value = converter(value)
+            return value
+
+    def _get_service_config(self, key, service_type):
+        config_dict = self.config.get(key)
+        if not config_dict:
+            return None
+        if not isinstance(config_dict, dict):
+            return config_dict
+
+        for st in self._service_type_manager.get_all_types(service_type):
+            if st in config_dict:
+                return config_dict[st]
+
     def get_interface(self, service_type=None):
-        key = _make_key('interface', service_type)
-        interface = self.config.get('interface')
-        return self.config.get(key, interface)
+        return self._get_config(
+            'interface', service_type, fallback_to_unprefixed=True)
 
     def get_api_version(self, service_type):
-        key = _make_key('api_version', service_type)
-        return self.config.get(key, None)
+        version = self._get_config('api_version', service_type)
+        if version:
+            try:
+                float(version)
+            except ValueError:
+                if 'latest' in version:
+                    warnings.warn(
+                        "You have a configured API_VERSION with 'latest' in"
+                        " it. In the context of openstacksdk this doesn't make"
+                        " any sense.")
+                return None
+        return version
+
+    def get_default_microversion(self, service_type):
+        return self._get_config('default_microversion', service_type)
 
     def get_service_type(self, service_type):
-        key = _make_key('service_type', service_type)
-        # Cinder did an evil thing where they defined a second service
-        # type in the catalog. Of course, that's insane, so let's hide this
-        # atrocity from the as-yet-unsullied eyes of our users.
-        # Of course, if the user requests a volumev2, that structure should
-        # still work.
-        # What's even more amazing is that they did it AGAIN with cinder v3
-        # And then I learned that mistral copied it.
-        # TODO(shade) This should get removed when we have os-service-types
-        # alias support landed in keystoneauth.
-        if service_type in ('volume', 'block-storage'):
-            vol_ver = self.get_api_version('volume')
-            if vol_ver and vol_ver.startswith('2'):
-                service_type = 'volumev2'
-            elif vol_ver and vol_ver.startswith('3'):
-                service_type = 'volumev3'
-        elif service_type == 'workflow':
-            wk_ver = self.get_api_version(service_type)
-            if wk_ver and wk_ver.startswith('2'):
-                service_type = 'workflowv2'
-        return self.config.get(key, service_type)
+        # People requesting 'volume' are doing so because os-client-config
+        # let them. What they want is block-storage, not explicitly the
+        # v1 of cinder. If someone actually wants v1, they'll have api_version
+        # set to 1, in which case block-storage will still work properly.
+        # Use service-types-manager to grab the official type name. _get_config
+        # will still look for config by alias, but starting with the official
+        # type will get us things in the right order.
+        if self._service_type_manager.is_known(service_type):
+            service_type = self._service_type_manager.get_service_type(
+                service_type)
+        return self._get_config(
+            'service_type', service_type, default=service_type)
 
     def get_service_name(self, service_type):
-        key = _make_key('service_name', service_type)
-        return self.config.get(key, None)
+        return self._get_config('service_name', service_type)
 
     def get_endpoint(self, service_type):
-        key = _make_key('endpoint_override', service_type)
-        old_key = _make_key('endpoint', service_type)
-        return self.config.get(key, self.config.get(old_key, None))
+        value = self._get_config('endpoint_override', service_type)
+        if not value:
+            value = self._get_config('endpoint', service_type)
+        if not value and self.config.get('auth_type') == 'none':
+            # If endpoint is given and we're using the none auth type,
+            # then the endpoint value is the endpoint_override for every
+            # service.
+            value = self.config.get('auth', {}).get('endpoint')
+        return value
+
+    def get_connect_retries(self, service_type):
+        return self._get_config('connect_retries', service_type,
+                                fallback_to_unprefixed=True,
+                                converter=int)
+
+    def get_status_code_retries(self, service_type):
+        return self._get_config('status_code_retries', service_type,
+                                fallback_to_unprefixed=True,
+                                converter=int)
 
     @property
     def prefer_ipv6(self):
@@ -278,7 +365,7 @@ class CloudRegion(object):
                 auth=self._auth,
                 verify=verify,
                 cert=cert,
-                timeout=self.config['api_timeout'],
+                timeout=self.config.get('api_timeout'),
                 discovery_cache=self._discovery_cache)
             self.insert_user_agent()
             # Using old keystoneauth with new os-client-config fails if
@@ -295,7 +382,7 @@ class CloudRegion(object):
         """Helper method to grab the service catalog."""
         return self._auth.get_access(self.get_session()).service_catalog
 
-    def _get_version_request(self, service_key, version):
+    def _get_version_request(self, service_type, version):
         """Translate OCC version args to those needed by ksa adapter.
 
         If no version is requested explicitly and we have a configured version,
@@ -306,6 +393,9 @@ class CloudRegion(object):
 
         If version is not set and we don't have a configured version, default
         to latest.
+
+        If version is set, contains a '.', and default_microversion is not
+        set, also pass it as a default microversion.
         """
         version_request = _util.VersionRequest()
         if version == 'latest':
@@ -313,20 +403,56 @@ class CloudRegion(object):
             return version_request
 
         if not version:
-            version = self.get_api_version(service_key)
+            version = self.get_api_version(service_type)
 
         # Octavia doens't have a version discovery document. Hard-code an
         # exception to this logic for now.
-        if not version and service_key not in ('load-balancer',):
+        if not version and service_type not in ('load-balancer',):
             version_request.max_api_version = 'latest'
         else:
             version_request.version = version
+
+        default_microversion = self.get_default_microversion(service_type)
+        implied_microversion = _get_implied_microversion(version)
+        if (implied_microversion and default_microversion
+                and implied_microversion != default_microversion):
+            raise exceptions.ConfigException(
+                "default_microversion of {default_microversion} was given"
+                " for {service_type}, but api_version looks like a"
+                " microversion as well. Please set api_version to just the"
+                " desired major version, or omit default_microversion".format(
+                    default_microversion=default_microversion,
+                    service_type=service_type))
+        if implied_microversion:
+            default_microversion = implied_microversion
+            # If we're inferring a microversion, don't pass the whole
+            # string in as api_version, since that tells keystoneauth
+            # we're looking for a major api version.
+            version_request.version = version[0]
+
+        version_request.default_microversion = default_microversion
+
         return version_request
 
+    def get_all_version_data(self, service_type):
+        # Seriously. Don't think about the existential crisis
+        # that is the next line. You'll wind up in cthulhu's lair.
+        service_type = self.get_service_type(service_type)
+        versions = self.get_session().get_all_version_data(
+            service_type=service_type,
+            interface=self.get_interface(service_type),
+            region_name=self.region_name,
+        )
+        region_versions = versions.get(self.region_name, {})
+        interface_versions = region_versions.get(
+            self.get_interface(service_type), {})
+        return interface_versions.get(service_type, [])
+
     def get_session_client(
-            self, service_key, version=None, constructor=adapter.Adapter,
+            self, service_type, version=None,
+            constructor=_adapter.OpenStackSDKAdapter,
             **kwargs):
-        """Return a prepped requests adapter for a given service.
+        """Return a prepped keystoneauth Adapter for a given service.
 
         This is useful for making direct requests calls against a
         'mounted' endpoint. That is, if you do:
@@ -339,78 +465,143 @@ class CloudRegion(object):
 
         and it will work like you think.
         """
-        version_request = self._get_version_request(service_key, version)
+        version_request = self._get_version_request(service_type, version)
 
-        return constructor(
+        kwargs.setdefault('connect_retries',
+                          self.get_connect_retries(service_type))
+        kwargs.setdefault('status_code_retries',
+                          self.get_status_code_retries(service_type))
+        endpoint_override = self.get_endpoint(service_type)
+        version = version_request.version
+        min_api_version = (
+            kwargs.pop('min_version', None) or version_request.min_api_version)
+        max_api_version = (
+            kwargs.pop('max_version', None) or version_request.max_api_version)
+        # Older neutron has inaccessible discovery document. Nobody noticed
+        # because neutronclient hard-codes an append of v2.0. YAY!
+        if service_type == 'network':
+            version = None
+            min_api_version = None
+            max_api_version = None
+            if endpoint_override is None:
+                network_adapter = constructor(
+                    session=self.get_session(),
+                    service_type=self.get_service_type(service_type),
+                    service_name=self.get_service_name(service_type),
+                    interface=self.get_interface(service_type),
+                    region_name=self.region_name,
+                )
+                network_endpoint = network_adapter.get_endpoint()
+                if not network_endpoint.rstrip().rsplit('/')[-1] == 'v2.0':
+                    if not network_endpoint.endswith('/'):
+                        network_endpoint += '/'
+                    network_endpoint = urllib.parse.urljoin(
+                        network_endpoint, 'v2.0')
+                endpoint_override = network_endpoint
+
+        client = constructor(
             session=self.get_session(),
-            service_type=self.get_service_type(service_key),
-            service_name=self.get_service_name(service_key),
-            interface=self.get_interface(service_key),
+            service_type=self.get_service_type(service_type),
+            service_name=self.get_service_name(service_type),
+            interface=self.get_interface(service_type),
             region_name=self.region_name,
-            version=version_request.version,
-            min_version=version_request.min_api_version,
-            max_version=version_request.max_api_version,
+            version=version,
+            min_version=min_api_version,
+            max_version=max_api_version,
+            endpoint_override=endpoint_override,
+            default_microversion=version_request.default_microversion,
+            rate_limit=self.get_rate_limit(service_type),
+            concurrency=self.get_concurrency(service_type),
             **kwargs)
-
-    def _get_highest_endpoint(self, service_types, kwargs):
-        session = self.get_session()
-        for service_type in service_types:
-            kwargs['service_type'] = service_type
-            try:
-                # Return the highest version we find that matches
-                # the request
-                return session.get_endpoint(**kwargs)
-            except keystoneauth1.exceptions.catalog.EndpointNotFound:
-                pass
+        if version_request.default_microversion:
+            default_microversion = version_request.default_microversion
+            info = client.get_endpoint_data()
+            if not discover.version_between(
+                    info.min_microversion,
+                    info.max_microversion,
+                    default_microversion
+            ):
+                if self.get_default_microversion(service_type):
+                    raise exceptions.ConfigException(
+                        "A default microversion for service {service_type} of"
+                        " {default_microversion} was requested, but the cloud"
+                        " only supports a minimum of {min_microversion} and"
+                        " a maximum of {max_microversion}.".format(
+                            service_type=service_type,
+                            default_microversion=default_microversion,
+                            min_microversion=discover.version_to_string(
+                                info.min_microversion),
+                            max_microversion=discover.version_to_string(
+                                info.max_microversion)))
+                else:
+                    raise exceptions.ConfigException(
+                        "A default microversion for service {service_type} of"
+                        " {default_microversion} was requested, but the cloud"
+                        " only supports a maximum of"
+                        " only supports a minimum of {min_microversion} and"
+                        " a maximum of {max_microversion}. The default"
+                        " microversion was set because a microversion"
+                        " formatted version string, '{api_version}', was"
+                        " passed for the api_version of the service. If it"
+                        " was not intended to set a default microversion"
+                        " please remove anything other than an integer major"
+                        " version from the version setting for"
+                        " the service.".format(
+                            service_type=service_type,
+                            api_version=self.get_api_version(service_type),
+                            default_microversion=default_microversion,
+                            min_microversion=discover.version_to_string(
+                                info.min_microversion),
+                            max_microversion=discover.version_to_string(
+                                info.max_microversion)))
+        return client
 
     def get_session_endpoint(
-            self, service_key, min_version=None, max_version=None):
+            self, service_type, min_version=None, max_version=None):
         """Return the endpoint from config or the catalog.
 
         If a configuration lists an explicit endpoint for a service,
         return that. Otherwise, fetch the service catalog from the
         keystone session and return the appropriate endpoint.
 
-        :param service_key: Generic key for service, such as 'compute' or
-                            'network'
-
+        :param service_type: Official service type of service
         """
 
-        override_endpoint = self.get_endpoint(service_key)
+        override_endpoint = self.get_endpoint(service_type)
         if override_endpoint:
             return override_endpoint
-        endpoint = None
-        kwargs = {
-            'service_name': self.get_service_name(service_key),
-            'region_name': self.region_name
-        }
-        kwargs['interface'] = self.get_interface(service_key)
-        if service_key == 'volume' and not self.get_api_version('volume'):
-            # If we don't have a configured cinder version, we can't know
-            # to request a different service_type
-            min_version = float(min_version or 1)
-            max_version = float(max_version or 3)
-            min_major = math.trunc(float(min_version))
-            max_major = math.trunc(float(max_version))
-            versions = range(int(max_major) + 1, int(min_major), -1)
-            service_types = []
-            for version in versions:
-                if version == 1:
-                    service_types.append('volume')
-                else:
-                    service_types.append('volumev{v}'.format(v=version))
-        else:
-            service_types = [self.get_service_type(service_key)]
-        endpoint = self._get_highest_endpoint(service_types, kwargs)
+
+        service_name = self.get_service_name(service_type)
+        interface = self.get_interface(service_type)
+        session = self.get_session()
+        # Do this as kwargs because of os-client-config unittest mocking
+        version_kwargs = {}
+        if min_version:
+            version_kwargs['min_version'] = min_version
+        if max_version:
+            version_kwargs['max_version'] = max_version
+        try:
+            # Return the highest version we find that matches
+            # the request
+            endpoint = session.get_endpoint(
+                service_type=service_type,
+                region_name=self.region_name,
+                interface=interface,
+                service_name=service_name,
+                **version_kwargs
+            )
+        except keystoneauth1.exceptions.catalog.EndpointNotFound:
+            endpoint = None
         if not endpoint:
             self.log.warning(
                 "Keystone catalog entry not found ("
-                "service_type=%s,service_name=%s"
+                "service_type=%s,service_name=%s,"
                 "interface=%s,region_name=%s)",
-                service_key,
-                kwargs['service_name'],
-                kwargs['interface'],
-                kwargs['region_name'])
+                service_type,
+                service_name,
+                interface,
+                self.region_name,
+            )
         return endpoint
 
     def get_cache_expiration_time(self):
@@ -547,3 +738,11 @@ class CloudRegion(object):
 
     def get_password_callback(self):
         return self._password_callback
+
+    def get_rate_limit(self, service_type=None):
+        return self._get_service_config(
+            'rate_limit', service_type=service_type)
+
+    def get_concurrency(self, service_type=None):
+        return self._get_service_config(
+            'concurrency', service_type=service_type)
