@@ -17,10 +17,12 @@ from openstack.load_balancer.v2 import listener as _listener
 from openstack.load_balancer.v2 import load_balancer as _lb
 from openstack.load_balancer.v2 import member as _member
 from openstack.load_balancer.v2 import pool as _pool
-from openstack import proxy2
+from openstack.load_balancer.v2 import quota as _quota
+from openstack import proxy
+from openstack import resource
 
 
-class Proxy(proxy2.BaseProxy):
+class Proxy(proxy.Proxy):
 
     def create_load_balancer(self, **attrs):
         """Create a new load balancer from attributes
@@ -55,7 +57,8 @@ class Proxy(proxy2.BaseProxy):
         """
         return self._list(_lb.LoadBalancer, paginated=True, **query)
 
-    def delete_load_balancer(self, load_balancer, ignore_missing=True):
+    def delete_load_balancer(self, load_balancer, ignore_missing=True,
+                             cascade=False):
         """Delete a load balancer
 
         :param load_balancer: The load_balancer can be either the name or a
@@ -66,9 +69,13 @@ class Proxy(proxy2.BaseProxy):
             the load balancer does not exist.
             When set to ``True``, no exception will be set when attempting to
             delete a nonexistent load balancer.
+        :param bool cascade: If true will delete all child objects of
+            the load balancer.
 
         :returns: ``None``
         """
+        load_balancer = self._get_resource(_lb.LoadBalancer, load_balancer)
+        load_balancer.cascade = cascade
         return self._delete(_lb.LoadBalancer, load_balancer,
                             ignore_missing=ignore_missing)
 
@@ -101,6 +108,13 @@ class Proxy(proxy2.BaseProxy):
         """
         return self._update(_lb.LoadBalancer, load_balancer, **attrs)
 
+    def wait_for_load_balancer(self, name_or_id, status='ACTIVE',
+                               failures=['ERROR'], interval=2, wait=300):
+        lb = self._find(_lb.LoadBalancer, name_or_id, ignore_missing=False)
+
+        return resource.wait_for_status(self, lb, status, failures, interval,
+                                        wait, attribute='provisioning_status')
+
     def create_listener(self, **attrs):
         """Create a new listener from attributes
 
@@ -570,7 +584,7 @@ class Proxy(proxy2.BaseProxy):
         :rtype: :class:`~openstack.load_balancer.v2.l7_rule.L7Rule`
         """
         l7policyobj = self._get_resource(_l7policy.L7Policy, l7_policy)
-        return self._create(_l7rule.L7Rule, l7_policy_id=l7policyobj.id,
+        return self._create(_l7rule.L7Rule, l7policy_id=l7policyobj.id,
                             **attrs)
 
     def delete_l7_rule(self, l7rule, l7_policy, ignore_missing=True):
@@ -592,7 +606,7 @@ class Proxy(proxy2.BaseProxy):
         """
         l7policyobj = self._get_resource(_l7policy.L7Policy, l7_policy)
         self._delete(_l7rule.L7Rule, l7rule, ignore_missing=ignore_missing,
-                     l7_policy_id=l7policyobj.id)
+                     l7policy_id=l7policyobj.id)
 
     def find_l7_rule(self, name_or_id, l7_policy, ignore_missing=True):
         """Find a single l7rule
@@ -613,7 +627,7 @@ class Proxy(proxy2.BaseProxy):
         l7policyobj = self._get_resource(_l7policy.L7Policy, l7_policy)
         return self._find(_l7rule.L7Rule, name_or_id,
                           ignore_missing=ignore_missing,
-                          l7_policy_id=l7policyobj.id)
+                          l7policy_id=l7policyobj.id)
 
     def get_l7_rule(self, l7rule, l7_policy):
         """Get a single l7rule
@@ -631,7 +645,7 @@ class Proxy(proxy2.BaseProxy):
         """
         l7policyobj = self._get_resource(_l7policy.L7Policy, l7_policy)
         return self._get(_l7rule.L7Rule, l7rule,
-                         l7_policy_id=l7policyobj.id)
+                         l7policy_id=l7policyobj.id)
 
     def l7_rules(self, l7_policy, **query):
         """Return a generator of l7rules
@@ -647,7 +661,7 @@ class Proxy(proxy2.BaseProxy):
         """
         l7policyobj = self._get_resource(_l7policy.L7Policy, l7_policy)
         return self._list(_l7rule.L7Rule, paginated=True,
-                          l7_policy_id=l7policyobj.id, **query)
+                          l7policy_id=l7policyobj.id, **query)
 
     def update_l7_rule(self, l7rule, l7_policy, **attrs):
         """Update a l7rule
@@ -666,4 +680,69 @@ class Proxy(proxy2.BaseProxy):
         """
         l7policyobj = self._get_resource(_l7policy.L7Policy, l7_policy)
         return self._update(_l7rule.L7Rule, l7rule,
-                            l7_policy_id=l7policyobj.id, **attrs)
+                            l7policy_id=l7policyobj.id, **attrs)
+
+    def quotas(self, **query):
+        """Return a generator of quotas
+
+        :param dict query: Optional query parameters to be sent to limit
+                           the resources being returned. Currently no query
+                           parameter is supported.
+
+        :returns: A generator of quota objects
+        :rtype: :class:`~openstack.load_balancer.v2.quota.Quota`
+        """
+        return self._list(_quota.Quota, paginated=False, **query)
+
+    def get_quota(self, quota):
+        """Get a quota
+
+        :param quota: The value can be the ID of a quota or a
+                      :class:`~openstack.load_balancer.v2.quota.Quota`
+                      instance. The ID of a quota is the same as the project
+                      ID for the quota.
+
+        :returns: One :class:`~openstack.load_balancer.v2.quota.Quota`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+                 when no resource can be found.
+        """
+        return self._get(_quota.Quota, quota)
+
+    def update_quota(self, quota, **attrs):
+        """Update a quota
+
+        :param quota: Either the ID of a quota or a
+                      :class:`~openstack.load_balancer.v2.quota.Quota`
+                      instance. The ID of a quota is the same as the
+                      project ID for the quota.
+        :param dict attrs: The attributes to update on the quota represented
+                           by ``quota``.
+
+        :returns: The updated quota
+        :rtype: :class:`~openstack.load_balancer.v2.quota.Quota`
+        """
+        return self._update(_quota.Quota, quota, **attrs)
+
+    def get_quota_default(self):
+        """Get a default quota
+
+        :returns: One :class:`~openstack.load_balancer.v2.quota.QuotaDefault`
+        """
+        return self._get(_quota.QuotaDefault, requires_id=False)
+
+    def delete_quota(self, quota, ignore_missing=True):
+        """Delete a quota (i.e. reset to the default quota)
+
+        :param quota: The value can be either the ID of a quota or a
+                      :class:`~openstack.load_balancer.v2.quota.Quota`
+                      instance. The ID of a quota is the same as the
+                      project ID for the quota.
+        :param bool ignore_missing: When set to ``False``
+                    :class:`~openstack.exceptions.ResourceNotFound` will be
+                    raised when quota does not exist.
+                    When set to ``True``, no exception will be set when
+                    attempting to delete a nonexistent quota.
+
+        :returns: ``None``
+        """
+        self._delete(_quota.Quota, quota, ignore_missing=ignore_missing)
