@@ -10,32 +10,31 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from openstack.network import network_service
-from openstack.network.v2 import tag
-from openstack import resource2 as resource
+from openstack import resource
 
 
-class Port(resource.Resource, tag.TagMixin):
+class Port(resource.Resource, resource.TagMixin):
     resource_key = 'port'
     resources_key = 'ports'
     base_path = '/ports'
-    service = network_service.NetworkService()
 
     # capabilities
     allow_create = True
-    allow_get = True
-    allow_update = True
+    allow_fetch = True
+    allow_commit = True
     allow_delete = True
     allow_list = True
 
     # NOTE: we skip query on list or datetime fields for now
     _query_mapping = resource.QueryParameters(
+        'binding:host_id', 'binding:profile', 'binding:vif_details',
+        'binding:vif_type', 'binding:vnic_type',
         'description', 'device_id', 'device_owner', 'fixed_ips', 'ip_address',
         'mac_address', 'name', 'network_id', 'status', 'subnet_id',
         is_admin_state_up='admin_state_up',
         is_port_security_enabled='port_security_enabled',
         project_id='tenant_id',
-        **tag.TagMixin._tag_query_parameters
+        **resource.TagMixin._tag_query_parameters
     )
 
     # Properties
@@ -79,12 +78,12 @@ class Port(resource.Resource, tag.TagMixin):
     device_owner = resource.Body('device_owner')
     #: DNS assignment for the port.
     dns_assignment = resource.Body('dns_assignment')
+    #: DNS domain assigned to the port.
+    dns_domain = resource.Body('dns_domain')
     #: DNS name for the port.
     dns_name = resource.Body('dns_name')
     #: Extra DHCP options.
     extra_dhcp_opts = resource.Body('extra_dhcp_opts', type=list)
-    #: IP addresses of an allowed address pair.
-    ip_address = resource.Body('ip_address')
     #: IP addresses for the port. Includes the IP address and subnet ID.
     fixed_ips = resource.Body('fixed_ips', type=list)
     #: The administrative state of the port, which is up ``True`` or
@@ -103,10 +102,9 @@ class Port(resource.Resource, tag.TagMixin):
     #: The ID of the project who owns the network. Only administrative
     #: users can specify a project ID other than their own.
     project_id = resource.Body('tenant_id')
-    #: The extra DHCP option name.
-    option_name = resource.Body('opt_name')
-    #: The extra DHCP option value.
-    option_value = resource.Body('opt_value')
+    #: Whether to propagate uplink status of the port. *Type: bool*
+    propagate_uplink_status = resource.Body('propagate_uplink_status',
+                                            type=bool)
     #: The ID of the QoS policy attached to the port.
     qos_policy_id = resource.Body('qos_policy_id')
     #: Revision number of the port. *Type: int*
@@ -116,11 +114,6 @@ class Port(resource.Resource, tag.TagMixin):
     security_group_ids = resource.Body('security_groups', type=list)
     #: The port status. Value is ``ACTIVE`` or ``DOWN``.
     status = resource.Body('status')
-    #: The ID of the subnet. If you specify only a subnet UUID, OpenStack
-    #: networking allocates an available IP from that subnet to the port.
-    #: If you specify both a subnet ID and an IP address, OpenStack networking
-    #: tries to allocate the address to the port.
-    subnet_id = resource.Body('subnet_id')
     #: Read-only. The trunk referring to this parent port and its subports.
     #: Present for trunk parent ports if ``trunk-details`` extension is loaded.
     #: *Type: dict with keys: trunk_id, sub_ports.
@@ -129,6 +122,3 @@ class Port(resource.Resource, tag.TagMixin):
     trunk_details = resource.Body('trunk_details', type=dict)
     #: Timestamp when the port was last updated.
     updated_at = resource.Body('updated_at')
-    #: A list of assocaited tags
-    #: *Type: list of tag strings*
-    tags = resource.Body('tags', type=list)
