@@ -38,12 +38,22 @@ def _disable_keep_alive(conn):
     sess = conn.config.get_session()
     sess.keep_alive = False
 
-IMAGE_NAME = _get_resource_value('image_name', 'cirros-0.3.5-x86_64-disk')
+
+IMAGE_NAME = _get_resource_value('image_name', 'cirros-0.3.6-x86_64-disk')
 FLAVOR_NAME = _get_resource_value('flavor_name', 'm1.small')
 
 
 class BaseFunctionalTest(base.TestCase):
 
+    @classmethod
+    def setUpClass(cls):
+        super(BaseFunctionalTest, cls).setUpClass()
+        # Defines default timeout for wait_for methods used
+        # in the functional tests
+        cls._wait_for_timeout = int(os.getenv(
+            'OPENSTACKSDK_FUNC_TEST_TIMEOUT',
+            300))
+
     def setUp(self):
         super(BaseFunctionalTest, self).setUp()
         self.conn = connection.Connection(config=TEST_CLOUD_REGION)
@@ -129,10 +139,12 @@ class BaseFunctionalTest(base.TestCase):
         if not min_microversion:
             return
 
-        if not (data.min_microversion and data.max_microversion and
-                discover.version_between(data.min_microversion,
-                                         data.max_microversion,
-                                         min_microversion)):
+        if not (data.min_microversion
+                and data.max_microversion
+                and discover.version_between(
+                    data.min_microversion,
+                    data.max_microversion,
+                    min_microversion)):
             self.skipTest('Service {service_type} does not provide '
                           'microversion {ver}'.format(
                               service_type=service_type,
