@@ -22,8 +22,8 @@ from openstack import exceptions
 class Test_Exception(base.TestCase):
     def test_method_not_supported(self):
         exc = exceptions.MethodNotSupported(self.__class__, 'list')
-        expected = ('The list method is not supported for ' +
-                    'openstack.tests.unit.test_exceptions.Test_Exception')
+        expected = ('The list method is not supported for '
+                    + 'openstack.tests.unit.test_exceptions.Test_Exception')
         self.assertEqual(expected, str(exc))
 
 
