@@ -683,6 +683,48 @@ class TestResource(base.TestCase):
         }
         self.assertEqual(expected, res.to_dict())
 
+    def test_to_dict_nested(self):
+
+        class Test(resource.Resource):
+            foo = resource.Header('foo')
+            bar = resource.Body('bar')
+            a_list = resource.Body('a_list')
+
+        class Sub(resource.Resource):
+            sub = resource.Body('foo')
+
+        sub = Sub(id='ANOTHER_ID', foo='bar')
+
+        res = Test(
+            id='FAKE_ID',
+            bar=sub,
+            a_list=[sub])
+
+        expected = {
+            'id': 'FAKE_ID',
+            'name': None,
+            'location': None,
+            'foo': None,
+            'bar': {
+                'id': 'ANOTHER_ID',
+                'name': None,
+                'sub': 'bar',
+                'location': None,
+            },
+            'a_list': [{
+                'id': 'ANOTHER_ID',
+                'name': None,
+                'sub': 'bar',
+                'location': None,
+            }],
+        }
+        self.assertEqual(expected, res.to_dict())
+        a_munch = res.to_dict(_to_munch=True)
+        self.assertEqual(a_munch.bar.id, 'ANOTHER_ID')
+        self.assertEqual(a_munch.bar.sub, 'bar')
+        self.assertEqual(a_munch.a_list[0].id, 'ANOTHER_ID')
+        self.assertEqual(a_munch.a_list[0].sub, 'bar')
+
     def test_to_dict_no_body(self):
 
         class Test(resource.Resource):
